use crate::lib::ai::AI;
use crate::Point;
use crate::Screen;
use std::collections::HashMap;

use ncurses::*;

use downcast_rs::{impl_downcast, Downcast};

pub trait Entity: AI + Renderable + Downcast {}
impl_downcast!(Entity);

pub struct Ant {
    pub pos: Point,
    pub id: u32,
}

impl Ant {
    pub fn new(x: i32, y: i32, id: u32) -> Ant {
        Ant {
            pos: Point(x, y),
            id,
        }
    }
}

impl Entity for Ant {}

impl Renderable for Ant {
    fn representation(&self) -> &str {
        "o"
    }
    fn before_render(&self) {
        attron(A_BOLD());
    }

    fn after_render(&self) {
        attroff(A_BOLD());
    }
}

pub struct Queen {
    pub pos: Point,
    pub egg_count: u8,
    pub id: u32,
}

impl Renderable for Queen {
    fn representation(&self) -> &str {
        "q"
    }

    fn before_render(&self) {
        attron(A_BOLD());
    }

    fn after_render(&self) {
        attroff(A_BOLD());
    }
}

pub trait Renderable: AI {
    fn representation(&self) -> &str;
    fn before_render(&self) {}
    fn after_render(&self) {}
    fn render(&self, b: &Screen) {
        self.before_render();
        b.render(&self.get_position(), self.representation());
        self.after_render();
    }
}

impl Queen {
    pub fn new(x: i32, y: i32, id: u32) -> Queen {
        Queen {
            pos: Point(x, y),
            egg_count: 0,
            id,
        }
    }
}

#[derive(Debug)]
pub struct Egg {
    pub pos: Point,
    pub counter: u8,
    pub queen_id: u32,
    pub id: u32,
}

impl Renderable for Egg {
    fn representation(&self) -> &str {
        "e"
    }

    fn before_render(&self) {
        attron(A_BOLD());
    }

    fn after_render(&self) {
        attroff(A_BOLD());
    }
}

impl Egg {
    pub fn new(x: i32, y: i32, id: u32, queen_id: u32) -> Egg {
        Egg {
            pos: Point(x, y),
            counter: 10,
            id,
            queen_id,
        }
    }
}

impl Entity for Queen {}
impl Entity for Egg {}

pub struct Entities {
    pub data: HashMap<u32, Box<dyn Entity>>,
    id_counter: u32,
}

impl Entities {
    pub fn new() -> Entities {
        Entities {
            id_counter: 0,
            data: HashMap::new(),
        }
    }

    pub fn add_ant(&mut self, x: i32, y: i32) -> u32 {
        let a = Ant::new(x, y, self.id_counter);
        let id = a.id;
        self.data.insert(a.id, Box::new(a));
        self.id_counter += 1;
        id
    }

    pub fn add_queen(&mut self, x: i32, y: i32) -> u32 {
        let q = Queen::new(x, y, self.id_counter);
        let id = q.id;
        self.data.insert(q.id, Box::new(q));
        self.id_counter += 1;
        id
    }

    pub fn add_egg(&mut self, x: i32, y: i32, queen_id: u32) -> u32 {
        let e = Egg::new(x, y, self.id_counter, queen_id);
        let id = e.id;
        self.data.insert(e.id, Box::new(e));
        self.id_counter += 1;
        id
    }
}
